package api

import (
	"box-cost/db/model"
	"fmt"

	_ "image/gif"
	_ "image/jpeg"
	_ "image/png"

	"github.com/xuri/excelize/v2"
)

type PurchaseBillExcel struct {
	Offset int

	Row int

	Title string

	Excel *excelize.File

	SheetName string

	AlignCenterStyle int

	Content *model.PurchaseBill

	Signatures []*model.Signature
	IsPdf      string
}

func (b *PurchaseBillExcel) drawTitle() error {
	b.Row++
	startCell := fmt.Sprintf("A%d", b.Row)
	endCell := fmt.Sprintf("J%d", b.Row)
	marginLeft := excelize.PageMarginLeft(0.15)
	b.Excel.SetColWidth(b.SheetName, "A", "A", 16)
	b.Excel.SetColWidth(b.SheetName, "A", "B", 16)
	b.Excel.SetColWidth(b.SheetName, "C", "G", 9)
	b.Excel.SetColWidth(b.SheetName, "H", "I", 12)
	// b.Excel.SetColWidth(b.SheetName, "J", "J", 24.5)
	b.Excel.SetColWidth(b.SheetName, "J", "J", 16)
	b.Excel.SetPageMargins(b.SheetName, marginLeft)
	err := b.Excel.MergeCell(b.SheetName, startCell, endCell)
	if err != nil {
		return err
	}

	style, err := b.Excel.NewStyle(&excelize.Style{
		Alignment: &excelize.Alignment{Horizontal: "center", Vertical: "center"},
		Font:      &excelize.Font{Bold: true, Size: 18}})
	if err != nil {
		return err
	}
	err = b.Excel.SetCellStyle(b.SheetName, startCell, startCell, style)
	if err != nil {
		return err
	}
	b.Excel.SetRowHeight(b.SheetName, b.Row, 23)
	b.Excel.SetCellValue(b.SheetName, startCell, b.Title)
	return nil
}

func (b *PurchaseBillExcel) drawSubTitles() error {
	b.Row++
	styleLeft, err := b.Excel.NewStyle(&excelize.Style{
		Alignment: &excelize.Alignment{Horizontal: "left", Vertical: "center"},
		Font:      &excelize.Font{Size: 11}})
	if err != nil {
		return err
	}
	styleRight, err := b.Excel.NewStyle(&excelize.Style{
		Alignment: &excelize.Alignment{Horizontal: "right", Vertical: "center"},
		Font:      &excelize.Font{Size: 11}})
	if err != nil {
		return err
	}

	drawLeft := func(rowIndex int, value string) error {
		//左边1
		left1Cell := fmt.Sprintf("A%d", rowIndex)
		err = b.Excel.MergeCell(b.SheetName, left1Cell, fmt.Sprintf("G%d", rowIndex))
		if err != nil {
			return err
		}
		err = b.Excel.SetCellStyle(b.SheetName, left1Cell, left1Cell, styleLeft)
		if err != nil {
			return err
		}
		b.Excel.SetCellValue(b.SheetName, left1Cell, value)
		return nil
	}

	drawRight := func(rowIndex int, value string) error {
		right1Cell := fmt.Sprintf("H%d", rowIndex)
		err = b.Excel.MergeCell(b.SheetName, right1Cell, fmt.Sprintf("J%d", rowIndex))
		if err != nil {
			return err
		}
		err = b.Excel.SetCellStyle(b.SheetName, right1Cell, right1Cell, styleRight)
		if err != nil {
			return err
		}
		b.Excel.SetCellValue(b.SheetName, right1Cell, value)
		return nil
	}

	//第一行
	drawLeft(b.Row, "类别："+b.Content.Type)
	drawRight(b.Row, "单号："+b.Content.SerialNumber)
	b.Excel.SetRowHeight(b.SheetName, b.Row, 21)

	//第二行
	drawLeft(b.Row+1, "供应商名称："+b.Content.Supplier)
	timeformat := b.Content.CreateTime.Local().Format("2006年01月02号 15:04:05")
	drawRight(b.Row+1, "下单时间："+timeformat)
	b.Excel.SetRowHeight(b.SheetName, b.Row+1, 21)

	//第三行
	drawLeft(b.Row+2, "产品名称："+b.Content.ProductName)
	status := ""
	if b.Content.Status == "complete" {
		status = "已完成"
	}
	if b.Content.Status == "created" {
		status = "进行中"
	}
	drawRight(b.Row+2, "状态："+status)
	b.Excel.SetRowHeight(b.SheetName, b.Row+2, 21)

	return nil
}

func (b *PurchaseBillExcel) drawTableTitle() error {
	b.Row += 3

	var drawCol = func(prefix string, value string) error {
		left1Cell := fmt.Sprintf("%s%d", prefix, b.Row)
		left2Cell := fmt.Sprintf("%s%d", prefix, b.Row+1)

		err := b.Excel.MergeCell(b.SheetName, left1Cell, left2Cell)
		if err != nil {
			return err
		}
		err = b.Excel.SetCellStyle(b.SheetName, left1Cell, left2Cell, b.AlignCenterStyle)
		if err != nil {
			return err
		}

		return b.Excel.SetCellValue(b.SheetName, left1Cell, value)
	}

	var drawCol2 = func(prefix1 string, prefix2 string, value1 string, value2 string, value3 string) error {
		left1Cell := fmt.Sprintf("%s%d", prefix1, b.Row)
		left2Cell := fmt.Sprintf("%s%d", prefix2, b.Row)
		err := b.Excel.MergeCell(b.SheetName, left1Cell, left2Cell)
		if err != nil {
			return err
		}
		if err != nil {
			fmt.Println(err)
			return err
		}
		err = b.Excel.SetCellStyle(b.SheetName, left1Cell, left2Cell, b.AlignCenterStyle)
		if err != nil {
			return err
		}
		b.Excel.SetCellValue(b.SheetName, left1Cell, value1)

		val2Cel := fmt.Sprintf("%s%d", prefix1, b.Row+1)
		b.Excel.SetCellStyle(b.SheetName, val2Cel, val2Cel, b.AlignCenterStyle)
		b.Excel.SetCellValue(b.SheetName, val2Cel, value2)

		val3Cel := fmt.Sprintf("%s%d", prefix2, b.Row+1)
		b.Excel.SetCellStyle(b.SheetName, val3Cel, val3Cel, b.AlignCenterStyle)
		b.Excel.SetCellValue(b.SheetName, val3Cel, value3)
		return nil
	}
	unit1 := "-"
	unit2 := "-"
	if len(b.Content.Paper) > 0 {
		if b.Content.Paper[0].PriceUnit != "" {
			unit1 = b.Content.Paper[0].PriceUnit
		} else if b.Content.Paper[0].Price2Unit != "" {
			unit2 = b.Content.Paper[0].Price2Unit
		} else {
			unit1 = "元/张"
			unit2 = "元/吨"
		}
		if b.Content.Paper[0].PriceUnit != "" && b.Content.Paper[0].Price2Unit != "" {
			unit1 = b.Content.Paper[0].PriceUnit
			unit2 = b.Content.Paper[0].Price2Unit
		}
	}

	drawCol("A", "采购项目")
	drawCol("B", "规格(克)")
	drawCol2("C", "D", "尺寸(mm)", "长", "宽")
	drawCol("E", "下单数量")
	drawCol2("F", "G", "单价", unit1, unit2)
	drawCol("H", "预算金额")
	drawCol("I", "交货时间")
	drawCol("J", "备注")

	return nil
}

func (b *PurchaseBillExcel) drawTableContent() error {
	b.Row += 2

	var DrawRow = func(rowIndex int, values ...string) {
		charas := []string{"A", "B", "C", "D", "E", "F", "G", "H", "I", "J"}
		for i, c := range charas {
			v := ""
			if i < len(values) {
				v = values[i]
			}
			b.Excel.SetCellValue(b.SheetName, fmt.Sprintf("%s%d", c, rowIndex), v)
			val2Cel := fmt.Sprintf("%s%d", c, rowIndex)
			b.Excel.SetCellStyle(b.SheetName, val2Cel, val2Cel, b.AlignCenterStyle)
			// var magN float64 = 1
			// lenv := len([]rune(v))
			// magN = math.Ceil(float64(lenv) / 10)
			// if lenv%10 > 5 {
			// 	magN = math.Ceil(float64(lenv) / 10)

			// } else {
			// 	n := math.Floor(float64(lenv) / 10)
			// 	if n > 0 {
			// 		magN = n
			// 	}
			// }
			// b.Excel.SetRowHeight(b.SheetName, rowIndex, 21*magN)
			b.Excel.SetRowHeight(b.SheetName, rowIndex, 46)

		}
	}
	papers := b.Content.Paper
	if len(papers) > 0 {
		for _, paper := range papers {
			deliveryTime := paper.DeliveryTime.Local().Format("2006-01-02")

			realCount := ""
			realPrice := ""
			price := fmt.Sprintf("%.3f", paper.OrderPrice)
			price2 := fmt.Sprintf("%.3f", paper.Price2)

			// 预算金额
			budgetAmount := fmt.Sprintf("%.3f", paper.OrderPrice*float64(paper.OrderCount))
			b.FormatToEmpty(&budgetAmount)

			// 实际完成数
			realCount = fmt.Sprintf("%d", paper.ConfirmCount)
			b.FormatToEmpty(&realCount)

			// 实际金额
			realPrice = fmt.Sprintf("%.3f", paper.OrderPrice*float64(paper.ConfirmCount))
			b.FormatToEmpty(&realPrice)
			DrawRow(b.Row, paper.Name, paper.Norm, paper.Height, paper.Width, fmt.Sprintf("%d", paper.OrderCount), price, price2, budgetAmount, deliveryTime, paper.Remark)
			b.Row++
		}
	}

	return nil
}

func (b *PurchaseBillExcel) drawTableFooter() error {
	border := []excelize.Border{
		{Type: "top", Style: 1, Color: "000000"},
		{Type: "left", Style: 1, Color: "000000"},
		{Type: "right", Style: 1, Color: "000000"},
		{Type: "bottom", Style: 1, Color: "000000"},
	}

	styleLeft, _ := b.Excel.NewStyle(&excelize.Style{
		Alignment: &excelize.Alignment{Horizontal: "left", Vertical: "center"},
		Border:    border,
	})

	sendToStartCell := fmt.Sprintf("A%d", b.Row)
	sendToEndCell := fmt.Sprintf("G%d", b.Row)
	supplierStartCell := fmt.Sprintf("H%d", b.Row)
	supplierEndCell := fmt.Sprintf("J%d", b.Row)

	b.Excel.MergeCell(b.SheetName, sendToStartCell, sendToEndCell)
	b.Excel.SetCellStyle(b.SheetName, sendToStartCell, sendToEndCell, styleLeft)
	b.Excel.SetCellValue(b.SheetName, sendToStartCell, "送货地址："+b.Content.SendTo)
	b.Excel.MergeCell(b.SheetName, supplierStartCell, supplierEndCell)
	b.Excel.SetCellStyle(b.SheetName, supplierStartCell, supplierEndCell, styleLeft)
	b.Excel.SetCellValue(b.SheetName, supplierStartCell, "供应商签字：")
	b.Excel.SetRowHeight(b.SheetName, b.Row, 21)

	return nil
}

func (b *PurchaseBillExcel) drawTableSignature() error {
	b.Row += 2

	style1, _ := b.Excel.NewStyle(&excelize.Style{
		Alignment: &excelize.Alignment{Horizontal: "center", Vertical: "center"},
	})

	// 制单人
	billUserCellS := fmt.Sprintf("A%d", b.Row+1)
	billUserCellE := fmt.Sprintf("B%d", b.Row+1)
	b.Excel.MergeCell(b.SheetName, billUserCellS, billUserCellE)
	b.Excel.SetCellValue(b.SheetName, billUserCellS, "制单人："+b.Content.UserName)

	fontNum := "G"
	image1s := "H"
	image2s := "J"
	image1e := "I"
	image2e := "J"

	fontCell := fmt.Sprintf("%s%d", fontNum, b.Row)
	fontEndCell := fmt.Sprintf("%s%d", fontNum, b.Row+2)

	imageCell1 := fmt.Sprintf("%s%d", image1s, b.Row)
	imageEndCell1 := fmt.Sprintf("%s%d", image1e, b.Row+2)

	imageCell2 := fmt.Sprintf("%s%d", image2s, b.Row)
	imageEndCell2 := fmt.Sprintf("%s%d", image2e, b.Row+2)

	b.Excel.MergeCell(b.SheetName, fontCell, fontEndCell)
	b.Excel.SetCellStyle(b.SheetName, fontCell, fontCell, style1)
	b.Excel.SetCellValue(b.SheetName, fontCell, "审核签字：")

	// 签字图片1 I10-J12
	b.Excel.MergeCell(b.SheetName, imageCell1, imageEndCell1)
	b.Excel.SetCellStyle(b.SheetName, imageCell1, imageCell1, style1)
	b.Excel.SetCellValue(b.SheetName, imageCell1, "")

	// 签字图片2 K10-L12
	b.Excel.MergeCell(b.SheetName, imageCell2, imageEndCell2)
	b.Excel.SetCellStyle(b.SheetName, imageCell2, imageCell1, style1)
	b.Excel.SetCellValue(b.SheetName, imageCell2, "")

	if b.Content.Reviewed == 1 {
		imageCells := []string{imageCell1, imageCell2}
		if len(b.Signatures) > 0 {
			for k, sign := range b.Signatures {
				b.Excel.AddPicture(b.SheetName, imageCells[k], sign.Path, sign.Format)

			}
		}
	}

	b.Excel.SetRowHeight(b.SheetName, b.Row, 21)

	return nil
}

func (b *PurchaseBillExcel) Draws() {
	b.drawTitle()
	b.drawSubTitles()
	b.drawTableTitle()
	b.drawTableContent()
	b.drawTableFooter()
	b.drawTableSignature()
}

func NewPurchaseBill(f *excelize.File) *PurchaseBillExcel {

	border := []excelize.Border{
		{Type: "top", Style: 1, Color: "000000"},
		{Type: "left", Style: 1, Color: "000000"},
		{Type: "right", Style: 1, Color: "000000"},
		{Type: "bottom", Style: 1, Color: "000000"},
	}

	styleLeft, _ := f.NewStyle(&excelize.Style{
		Alignment: &excelize.Alignment{Horizontal: "center", Vertical: "center", WrapText: true},
		Border:    border,
		Font:      &excelize.Font{Size: 10},
	})

	b := &PurchaseBillExcel{
		Title:            "原材料采购单",
		SheetName:        "Sheet1",
		Excel:            f,
		Offset:           0,
		AlignCenterStyle: styleLeft,
		Signatures:       make([]*model.Signature, 0),
	}

	f.SetPageMargins(b.SheetName, excelize.PageMarginTop(1), excelize.PageMarginLeft(0), excelize.PageMarginRight(0))
	return b
}

func (b *PurchaseBillExcel) FormatToEmpty(str *string) {
	if *str == "0" || *str == "0.000" {
		*str = ""
	}

}

func (b *PurchaseBillExcel) SetContent(content interface{}) {

	b.Content = content.(*model.PurchaseBill)

}
func (b *PurchaseBillExcel) SetTitle(title string) {
	b.Title = title

}
func (b *PurchaseBillExcel) SetSheetName(name string) {
	b.SheetName = name

}
func (b *PurchaseBillExcel) SetRow(row int) {
	b.Row = row
}
func (b *PurchaseBillExcel) SetIsPdf(isPdf string) {
	b.IsPdf = isPdf
}
func (b *PurchaseBillExcel) GetRow() int {
	return b.Row
}

func (b *PurchaseBillExcel) SetSignatures(sign interface{}) {
	b.Signatures = sign.([]*model.Signature)

}
